/*
 * BlogSetupView.cpp
 *
 *  Created on: 17.12.2009
 *      Author: apoi
 */

#include <QAction>
#include <QPushButton>
#include <QDateTime>
#include <QMessageBox>

#include "BlogView.h"
#include "BlogSetupView.h"
#include "wpscrollarea.h"
#include "collectionlist.h"
#include "textlistitem.h"
#include "WPNetworkEngine.h"

const char* KBlogsStyleSheet = "background-color: #ECECEC; font-size: 6pt; margin: 5px 0px 3px 0px; color: #333333;";

BlogSetupView::BlogSetupView(BlogView *aParent) : QFrame(aParent), iParent(aParent)
{
    setWindowFlags(Qt::Window);
#ifdef Q_WS_MAEMO_5
    setAttribute(Qt::WA_Maemo5StackedWindow);
#endif
    
    setWindowTitle(tr("Blog list"));
    QVBoxLayout* layout = new QVBoxLayout(this);
    iScrollArea = new WPScrollArea;
    layout->setSpacing(0);
    layout->setContentsMargins(0, 0, 0, 0);
    layout->addWidget(iScrollArea);

    iScrollArea->setWidgetResizable(true);
    iScrollArea->setFocusPolicy(Qt::NoFocus);
    iScrollArea->setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
    iScrollArea->setStyleSheet(KBlogsStyleSheet);
    
    // List of blogs
    iBlogList = new CollectionList(this, *iScrollArea, iParent->GetThemeData());
    iBlogList->SetTitle(tr("Blogs "));
    iScrollArea->setWidget(iBlogList);

    connect(iBlogList->iActionOpen, SIGNAL(triggered()), this, SLOT(Open()));
    connect(iBlogList->iActionEdit, SIGNAL(triggered()), this, SLOT(Edit()));
    connect(iBlogList->iActionDelete, SIGNAL(triggered()), this, SLOT(Delete()));

    // Button is destroyed when the list is reseted, and the
    // connection is removed at the same time
    QPushButton* addBlog = new QPushButton(tr("Add new blog..."), this);
    connect(addBlog, SIGNAL(clicked()), this, SLOT(AddNewBlog()));
    layout->addWidget(addBlog);

//#ifdef Q_OS_SYMBIAN
    QAction* rightSoftKey = new QAction(tr("Back"), this);
    connect(rightSoftKey, SIGNAL(triggered()), iParent, SLOT(ShowTabbedView()));
    rightSoftKey->setSoftKeyRole(QAction::NegativeSoftKey);
//    addAction(rightSoftKey);
/*#else
#ifndef Q_WS_MAEMO_5 // just to make this work on linux / mac environment
    QPushButton* backButton = new QPushButton(tr("Back"), this);
    connect(backButton, SIGNAL(clicked()), iParent, SLOT(ShowTabbedView()));
    layout->addWidget(backButton);
#endif
#endif*/

    // Finally add the data for WPCustomViewBasicData
    //
    iTitleText = tr("Blogs");
    iTabText = tr("Blogs");
    iHappyAction = rightSoftKey;
    iSadAction = rightSoftKey;
}

BlogSetupView::~BlogSetupView()
{
}

void BlogSetupView::UpdateData(const WPDataUsersBlogs& aUsersBlogs)
{
    iBlogList->ResetList();

    for (QListIterator<WPDataBlog *> i(aUsersBlogs.GetBlogs()); i.hasNext(); ) {
        AddBlog(*i.next());
    }

    iBlogList->SetTitle();
    iBlogList->AddStretch();
}

void BlogSetupView::AddBlog(const WPDataBlog& aBlog)
{
    QStringList list;
    //list << WPNetworkEngine::LocalizedTime(aBlog.i);
    // TODO get the numbers by looping through the blog data

    int draftPosts = 0;
    int unapprovedComments = 0;
    QDateTime latestActivity = QDateTime::fromString("19700101T00:00:00", "yyyyMMddTHH:mm:ss");
    QDateTime comparisonDate;

    for (QListIterator<WPPost> i = aBlog.GetPosts(); i.hasNext(); ) {
        const WPPost post = i.next();

        comparisonDate = QDateTime::fromString(post->iDateCreated, "yyyyMMddTHH:mm:ss");

        if (comparisonDate > latestActivity)
            latestActivity = comparisonDate;

        if (post->IsDraft())
            draftPosts++;
    }

    for (QListIterator<WPComment> j = aBlog.GetComments(); j.hasNext(); )
    {
        WPComment comment = j.next();

        comparisonDate = QDateTime::fromString(comment->iDateCreatedGmt);
        if (comparisonDate > latestActivity)
            latestActivity = comparisonDate;
        if (comment->IsUnapproved())
            unapprovedComments++;
    }

    QString status;
    if (unapprovedComments == 1)
        status += tr("1 unapproved comment");
    else if (unapprovedComments > 1)
        status += QString::number(unapprovedComments) + tr(" unapproved comments");
    if (draftPosts > 0)
    {
        if (!status.isEmpty())
            status += ", ";
        if (draftPosts == 1)
            status += tr("1 draft");
        else
            status += QString::number(draftPosts) + tr(" drafts");
    }
    list << status;
    list << tr("Latest activity: ") + WPNetworkEngine::LocalizedTime(latestActivity);

    iBlogList->AddCustomItem(aBlog.LocalId(), aBlog.iBlogName, QString(), list);
}

void BlogSetupView::keyReleaseEvent(QKeyEvent * event)
{
    qDebug("WP::BlogSetupView::keyPressEvent Key event %d", event->key());
    if (event->key() == Qt::Key_Context1 && iHappyAction)
    {
        iHappyAction->trigger();
        event->accept();
    }
    else if (event->key() == Qt::Key_Context2 && iSadAction)
    {
        iSadAction->trigger();
        event->accept();
    }
    else
    {
        event->ignore();
    }
}

void BlogSetupView::AddNewBlog()
{
    iParent->ShowNewBlogView();
}

void BlogSetupView::Open()
{
    iParent->OpenBlog(iBlogList->iCurrentItem->ItemId());
#ifdef Q_WS_MAEMO_5
    close();
#endif
}

void BlogSetupView::Edit()
{
    iParent->ShowEditBlogView(iBlogList->iCurrentItem->ItemId());
}

void BlogSetupView::Delete()
{
    QMessageBox box;
    box.setText(tr("Are you sure?"));
    box.setStandardButtons(QMessageBox::Yes | QMessageBox::No);
    box.setEscapeButton(QMessageBox::No);
    box.setDefaultButton(QMessageBox::No);
    const int result = box.exec();
    if (result == QMessageBox::No)
        return;
    if (iParent->DeleteBlog(iBlogList->iCurrentItem->ItemId()))
    {
        iBlogList->RemoveItem(iBlogList->iCurrentItem);
    }
}

void BlogSetupView::setFocus()
{
    iBlogList->setFocus();
}
