/*
 * BlogView.cpp
 *
 *  Created on: 5.11.2009
 *      Author: thyv
 */

#include <QDebug>
#include <QMessageBox>
#include <QMenu>
#include <QMenuBar>
#include <QAction>
#include <QStackedWidget>
#include <QFileSystemModel>
#include <QDesktopWidget>
#include <QResizeEvent>
#include <QGraphicsScene>
#include <QGraphicsView>
#include <QGraphicsLinearLayout>
#include <QLabel>
#include <QGraphicsProxyWidget>

#ifdef Q_OS_SYMBIAN
#include <aknappui.h>
#include <avkon.rsg>
#endif

#include "busyindicator.h"
#include "setupblog.h"
#include "BlogSetupView.h"
#include "BlogView.h"
#include "PostsView.h"
#include "PagesView.h"
#include "commentsview.h"
#include "commentsgraphicsview.h"
#include "addpostview.h"
#include "SinglePostView.h"
#include "CategoryView.h"
#include "writepageview.h"
#include "WPNetworkEngine.h"
#include "textlistitem.h"
#include "ImageSelectionList.h"
#include "ProgressDialog.h"
#include "DialogController.h"
#include "WPLogger.h"
#include "statisticsview.h"
#include "apikeydialog.h"
#include "wptabbargraphicswidget.h"
#include "enume/pannableview.h"
#include "wpmenugraphicswidget.h"
#include "wpstackedgraphicswidget.h"
#include "wpstatusbargraphicswidget.h"

BlogView::BlogView() :
    iNetworkEngine(NULL),
    iCurrentView(ETabbedView)
{
    // Workaround for issue of starting the app in landscape mode.
    // See the resizeEvent for more.
  /*#ifdef Q_WS_MAEMO_5
    setAttribute(Qt::WA_Maemo5StackedWindow);
    setAttribute(Qt::WA_Maemo5LandscapeOrientation, true);
    #endif*/
    iLandscapeSizeWorkaround = false;
    iBusyIndicator = NULL;
    iView = NULL;
    iScene = NULL;
    iLayout = NULL;
    iHorizontalLayout = NULL;
    iNetworkEngine = NULL;
    iThemeData = NULL;
    iImageSelectionList = NULL;

    // ThemeData stores common brushes and pixmaps so that
    // they don't have to be kept in memory by each object separately
    //
    iThemeData = new ThemeData;
    iThemeData->Init();

    // Initialize secondary views.
    // TODO: move all the options-views to be built later.
    //
    iBlogView = new BlogSetupView(this);
    iSetupBlog = new SetupBlog(this);
    iWritePageView = new WritePageView(this);
    iAddPostView = new AddPostView(this);
    iSinglePostView = new SinglePostView(*this);
    iDialogController = new DialogController(this);
    apiKeyDialog = new ApiKeyDialog();
    iStatisticsView = new StatisticsView(this);

/*#ifdef Q_WS_MAEMO_5
    iCategoryView = new CategoryView(this, iAddPostView);
#else*/
    iCategoryView = new CategoryView(this);
//#endif

    iViewTitle << tr("Edit blog")
            << tr("Blogs")
            << tr("Edit page")
            << tr("Edit post")
            << tr("Post")
            << tr("Categories")
            << tr("Statistics")
            << tr("WordPress")
            << tr("Images");

    // The order in which the stacked widget is initialized
    // is significant -- the enums have to match
    iStackedWidget = new QStackedWidget(this);

    //#ifndef Q_WS_MAEMO_5
    //iStackedWidget->insertWidget(ESetupBlogView, iSetupBlog);
    //iStackedWidget->insertWidget(EBlogListView, iBlogView);
    //iStackedWidget->insertWidget(EWritePageView, iWritePageView);
    //iStackedWidget->insertWidget(EAddPostView, iAddPostView);
    //iStackedWidget->insertWidget(ESinglePostView, iSinglePostView);
    //iStackedWidget->insertWidget(ECategoryView, iCategoryView);
    iStackedWidget->insertWidget(EStatisticsView, iStatisticsView);
    //#endif
    iScene = new QGraphicsScene(this);
    iView = new QGraphicsView(iScene, this);
    iLayoutWidget = new QGraphicsWidget();
    iHorizontalLayout = new QGraphicsLinearLayout(Qt::Horizontal, iLayoutWidget);
    iLayout = new QGraphicsLinearLayout(Qt::Vertical, iHorizontalLayout);
    
    iCategoryProxy = iScene->addWidget(iCategoryView);
    iCategoryProxy->setFlag(QGraphicsItem::ItemAcceptsInputMethod);
    iCategoryProxy->hide();
    
    iSetupBlogProxy = iScene->addWidget(iSetupBlog);
    iSetupBlogProxy->setFlag(QGraphicsItem::ItemAcceptsInputMethod);
    iSetupBlogProxy->hide();
    
    iBlogListProxy = iScene->addWidget(iBlogView);
    iBlogListProxy->setFlag(QGraphicsItem::ItemAcceptsInputMethod);
    iBlogListProxy->hide();
    
    // busy spinner widget.
    iBusyIndicator = new BusyIndicator(iStackedWidget);
    
    // Status bar
    iTitleBar = new WPTitleBarGraphicsWidget(*this, iLayoutWidget, *iScene, iBusyIndicator);
    
    // Tabbed view
    iTabContentWidget = new WPStackedGraphicsWidget(iLayoutWidget);
    iTabView = new WPTabbarGraphicsView(*this,iLayoutWidget, iTabContentWidget, *iScene);

    //iCommentsGraphicsView = new CommentsGraphicsView(*this, this);
    iPannablePostsWidget = new PannableView(Qt::Vertical);
    iPostsView = new PostsView(this,*iView, iPannablePostsWidget);
    
    iPannablePagesWidget = new PannableView(Qt::Vertical);
    iPagesView = new PagesView(this,*iView, iPannablePagesWidget);
    
    iPannableCommentsWidget = new PannableView(Qt::Vertical);
    iCommentsView = new CommentsView(this,*iView, iPannableCommentsWidget);

    iMenu = new WPMenuGraphicsWidget(*this, iLayoutWidget, *iScene);

    iTabView->addTab(iPannableCommentsWidget, tr("Comments"), ThemeData::EComments);
    iTabView->addTab(iPannablePostsWidget, tr("Posts"), ThemeData::EPosts);
    iTabView->addTab(iPannablePagesWidget, tr("Pages"), ThemeData::EPages);

    iStackedWidget->insertWidget(ETabbedView, iView);

    // Data model
    iNetworkEngine = new WPNetworkEngine();
    iBlogs = new WPDataUsersBlogs(iNetworkEngine);
    iGravatars = new WPDataGravatars(iNetworkEngine);

    // general data events
    // XXX: these never show up from LoadLocalData()
    connect(
        iBlogs,           SIGNAL(DataError(const QString&)), 
        iDialogController,  SLOT(ShowErrorNote(const QString&))
    );

    connect(
        iBlogs, SIGNAL(GetUserBlogsCompleted(const WPDataBlog&)), 
        this,     SLOT(ShowTabbedView())
    );

    // Progress dialog
    connect(
        iNetworkEngine, SIGNAL(TaskStarted(TWPNetworkEngineCommand)),
        iDialogController, SLOT(TaskStarted(TWPNetworkEngineCommand))
    );

    connect(
        iNetworkEngine, SIGNAL(TaskFinished(TWPNetworkEngineCommand)), 
        iDialogController, SLOT(TaskFinished(TWPNetworkEngineCommand))
    );

    connect(
        iNetworkEngine, SIGNAL(TaskFailed (TWPNetworkEngineCommand, int, QString)),
        iDialogController, SLOT(TaskFailed (TWPNetworkEngineCommand, int, QString))
    );

    // New data from network, update views
    connect(iNetworkEngine, SIGNAL(UpdateCategoryData(const WPDataBlog&)), iCategoryView, SLOT(UpdateCategoryData(const WPDataBlog&)));
    connect(iNetworkEngine, SIGNAL(UpdateStats(const WPDataStats&)), iStatisticsView, SLOT(updateStats(const WPDataStats&)));
    connect(iNetworkEngine, SIGNAL(UpdateStatsFailed(const QString&)), iStatisticsView, SLOT(updateStatsFailed(const QString&)));
   
    // data changed, update views
    connect(
        iBlogs,         SIGNAL(UpdateBlogs(const WPDataUsersBlogs &)),
        iBlogView,        SLOT(UpdateData(const WPDataUsersBlogs &))
    );

    connect(
        iBlogs,         SIGNAL(UpdatePageDataFull(const WPDataBlog &)), 
        iPagesView,       SLOT(UpdatePageData(const WPDataBlog &))
    );

    connect(
        iBlogs,         SIGNAL(UpdatePostDataFull(const WPDataBlog &)), 
        iPostsView,       SLOT(UpdatePostData(const WPDataBlog &))
    );

    connect(
        iBlogs,         SIGNAL(UpdateCommentDataFull(const WPDataBlog &)),
        iCommentsView,    SLOT(UpdateCommentsData(const WPDataBlog &))
    );
   
    // SinglePostView has both post and comments 
    connect(
        iBlogs,         SIGNAL(UpdatePostData(const WPDataBlog &)), 
        iSinglePostView,  SLOT(UpdateData(const WPDataBlog &))
    );

    connect(
        iBlogs,         SIGNAL(UpdateCommentData(const WPDataBlog &)), 
        iSinglePostView,  SLOT(UpdateData(const WPDataBlog &))
    );

    connect(
        iBlogs,         SIGNAL(UpdateCommentDataFull(const WPDataBlog &)), 
        iSinglePostView,  SLOT(UpdateData(const WPDataBlog &))
    );
    
    /* XXX:
    connect(
        iBlogs,         SIGNAL(UpdateCommentsData(const WPDataBlog&)), 
        iPostsView,       SLOT(UpdateCommentsData(const WPDataBlog&))
    );
    */

    // XXX: UpdateCommentsDataLight -> iPostsView?
    
    // Connect page view context menu signals
    connect(iPagesView, SIGNAL(OpenClicked(WPPage)), this, SLOT(ShowEditPageView(WPPage)));
    connect(iPagesView, SIGNAL(EditClicked(WPPage)), this, SLOT(ShowEditPageView(WPPage)));

    // Connect post view context menu signals
    connect(iPostsView, SIGNAL(OpenClicked(WPPost)), this, SLOT(ShowSinglePostView(WPPost)));
    connect(iPostsView, SIGNAL(EditClicked(WPPost)), this, SLOT(ShowEditPostView(WPPost)));
    connect(iPostsView, SIGNAL(DeleteClicked(WPPost)), this, SLOT(DeletePost(WPPost)));
    connect(
        iCategoryView, SIGNAL(NewCategory(QString)), 
        iNetworkEngine, SLOT(NewCategory(QString))
    );

    // Post editing view
    connect(iAddPostView, SIGNAL(ShowErrorNote(const QString&)), iDialogController, SLOT(ShowErrorNote(const QString&)));
    connect(iWritePageView, SIGNAL(ShowErrorNote(const QString&)), iDialogController, SLOT(ShowErrorNote(const QString&)));

    connect(
            apiKeyDialog, SIGNAL(setApiKey(const QString&)), 
            this, SLOT(SetApiKey(const QString&))
    );
    
    
    iScene->addItem(iLayoutWidget);
    iLayoutWidget->setLayout(iHorizontalLayout);

    iTitleBar->SetText(tr("Wordpress"));
    iScene->addItem(iTitleBar);
    iLayout->addItem(iTitleBar);
    
    iLayout->setSpacing(0);
    iLayout->setContentsMargins(0,0,0,0);
    iHorizontalLayout->setSpacing(0);
    iHorizontalLayout->setContentsMargins(0,0,0,0);
    iStackedWidget->setContentsMargins(0,0,0,0);
    
    iScene->addItem(iTabView);

    iPannablePagesWidget->setWidget(iPagesView);
    iPannablePagesWidget->enableScrollIndicator(true);
    iScene->addItem(iPannablePagesWidget);
    iPagesView->setContentsMargins(0.0, 0.0, 0.0, 0.0);

    iPannablePostsWidget->setWidget(iPostsView);
    iPannablePostsWidget->enableScrollIndicator(true);
    iScene->addItem(iPannablePostsWidget);
    iPostsView->setContentsMargins(0.0, 0.0, 0.0, 0.0);
    
    iPannableCommentsWidget->setWidget(iCommentsView);
    iPannableCommentsWidget->enableScrollIndicator(true);
    iScene->addItem(iPannableCommentsWidget);
    iCommentsView->setContentsMargins(0.0, 0.0, 0.0, 0.0);
    
    QRect rect(geometry());

    // Configure View
    iView->setDragMode(QGraphicsView::NoDrag);
    iView->setGeometry(rect);
    iView->setVerticalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
    iView->setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
    //QLinearGradient ql(0, 0, 300, 300);
    //ql.setColorAt(0.0, QColor(61, 165, 199));
    //ql.setColorAt(1.0, QColor(36, 80, 94));
    iView->setBackgroundBrush(QColor(236,236,236));
    iView->setCacheMode(QGraphicsView::CacheBackground);

    iScene->addItem(iTabContentWidget);
   
    iPannableCommentsWidget->hide();
    iPannablePagesWidget->hide();
    iPannablePostsWidget->hide();
    iTabContentWidget->AddWidget(iPannableCommentsWidget);
    iTabContentWidget->AddWidget(iPannablePostsWidget);
    iTabContentWidget->AddWidget(iPannablePagesWidget);

    // Embed single post view into graphics scene
    iSinglePostProxy = iScene->addWidget(iSinglePostView);
    iSinglePostProxy->setFlag(QGraphicsItem::ItemAcceptsInputMethod);
    iSinglePostProxy->hide();
    
    // Embed write page view into graphics scene
    iWritePageProxy = iScene->addWidget(iWritePageView);
    iWritePageProxy->setFlag(QGraphicsItem::ItemAcceptsInputMethod);
    // Embed add post view into graphics scene
    iAddPostProxy = iScene->addWidget(iAddPostView);
    iAddPostProxy->setFlag(QGraphicsItem::ItemAcceptsInputMethod);
    
    iTabContentWidget->AddWidget(iAddPostProxy);
    iTabContentWidget->AddWidget(iWritePageProxy);
    
    iImageSelectionProxy = new QGraphicsProxyWidget(iLayoutWidget);
    iScene->addItem(iImageSelectionProxy);
    
    // Setup main layout
    // Default to portrait.
    //
    iPortrait = true;
    iLayout->addItem(iTabView);
    iLayout->addItem(iTabContentWidget);
    iMenu->setMaximumHeight(MenuHeight);
    iLayout->addItem(iMenu);
    iHorizontalLayout->addItem(iLayout);
    iView->setScene(iScene);

    // If local data is found, go directly to the tabbed view and
    // refresh data from the network. Otherwise, go to blog setup.
    //
    // Note: Triggers refresh. UI must be built before call LoadLocalData().
    //
    if (iBlogs->LoadLocalData()) {
        setCentralWidget(iStackedWidget);
        ShowTabbedView();
    } else {
        // At first run, the setup blog view already contains default
        // values, so just switch to the view directly.
      /*#ifdef Q_WS_MAEMO_5
        // In Maemo, the central widget is first blog setup,
        // and will be changed to iStackedWidget once setup is done.
        iInitialSetupBlog = new SetupBlog(this);
        setCentralWidget(iInitialSetupBlog);
        SwitchView(EInitialSetupBlogView);
	#else*/
        setCentralWidget(iStackedWidget);
        ShowTabbedView();
        SwitchView(ESetupBlogView);
	//#endif
    }
    SetSoftKeys();
}

BlogView::~BlogView()
{
    delete iThemeData;

    if (iBlogs)
        iBlogs->SaveLocalData();

    delete iGravatars;
    delete iBlogs;
    delete iNetworkEngine;
}

void BlogView::keyReleaseEvent ( QKeyEvent * event  )
{
    //qDebug("WP::BlogView::keyReleaseEvent Key event %d", event->key());
    if (event->key() == Qt::Key_Context1 ||
        event->key() == Qt::Key_Context2 )
    {
        iMenu->showMenu();
        event->accept();
    }
    else if ( TabsActive() &&
            ( event->key() == Qt::Key_Left ||
              event->key() == Qt::Key_Right ) ) 
    {
        if (event->key() == Qt::Key_Left)
            iTabView->changeTabLeft();
        else
            iTabView->changeTabRight();
        event->accept();
    }
    else
    {
        event->ignore();
    }
}

void BlogView::LaunchBlogView()
{
    if (iCurrentView != ETabbedView)
    {
        iTitleBar->SetText(iBlogs->GetCurrentBlog() ? iBlogs->GetCurrentBlog()->iBlogName : tr("WordPress"));
        iMenu->setMode(WPMenuGraphicsWidget::EOptionsQuit);
        iTabContentWidget->SwitchPreviousWidget();
    }
}

void BlogView::SetSoftKeys()
{
    // Menu bar
    // TODO implement in a way that allows the same menu for all views
    //menuBar()->clear();
#ifdef Q_OS_MACX // Actions won't work in menubar on Mac OS X           
    //QMenu* menu = menuBar()->addMenu(tr("Menu"));
#else
    //QMenuBar* menu = menuBar();
#endif

    // Blogs submenu
    QAction* iManageBlogs = new QAction(tr("Blogs"), this);
    connect(iManageBlogs, SIGNAL(triggered()), this, SLOT(ShowBlogListView()));
    iMenu->addAction(iManageBlogs);

    // Other actions
    QAction* iActionWritePage = new QAction(tr("Write new page"), this);
    connect(iActionWritePage, SIGNAL(triggered()), this, SLOT(ShowWriteNewPageView()));
    iMenu->addAction(iActionWritePage);

    QAction* iActionWritePost = new QAction(tr("Write new post"), this);
    connect(iActionWritePost, SIGNAL(triggered()), this, SLOT(ShowWriteNewPostView()));
    iMenu->addAction(iActionWritePost);

    QAction* iActionShowStatistics = new QAction(tr("Statistics"), this);
    connect(iActionShowStatistics, SIGNAL(triggered()), this, SLOT(ShowStatisticsView()));
    iMenu->addAction(iActionShowStatistics);

    QAction* iActionRefresh = new QAction(tr("Refresh"), this);
    connect(iActionRefresh, SIGNAL(triggered()), this, SLOT(Refresh()));
    iMenu->addAction(iActionRefresh);

    QAction* iAbout = new QAction(tr("About"), this);
    connect(iAbout, SIGNAL(triggered()), this, SLOT(About()));
    iMenu->addAction(iAbout);

    QAction* iExit = new QAction(tr("Exit"), this);
    connect(iExit, SIGNAL(triggered()), this, SLOT(Exit()));
    iMenu->addAction(iExit);
}

void BlogView::ShowInLayout(QGraphicsWidget* aItem)
{
    const int KFirstContentElement = 1;
    const int KOptionalContentElement = 2;
    
    if (iLayout->itemAt(KFirstContentElement) == aItem)
    {
        // nothing to do
        return;
    }

    // Remove and hide current item(s).
    //
    if (iLayout->itemAt(KFirstContentElement) == iTabView)
    {
        // tab content also.
        iLayout->itemAt(KOptionalContentElement)->graphicsItem()->hide();
        iLayout->removeAt(KOptionalContentElement);
    }
    iLayout->itemAt(KFirstContentElement)->graphicsItem()->hide();
    iLayout->removeAt(KFirstContentElement);

    // Add and show new widget(s).
    //
    iLayout->insertItem(KFirstContentElement,aItem);
    aItem->show();
    
    if (aItem == iTabView)
    {
        iLayout->insertItem(KOptionalContentElement,iTabContentWidget);
        iTabContentWidget->show();
    }
}

bool BlogView::TabsActive()
{
    const int KFirstContentElement = 1;
    if (iLayout->itemAt(KFirstContentElement) == iTabView)
        return true;
    else
        return false;
}

void BlogView::ShowEditPageView(WPPage aPage)
{
    iWritePageView->EditPage(*iBlogs->GetCurrentBlog(), aPage);
    SwitchView(EWritePageView);
}

void BlogView::ShowWriteNewPageView()
{
    iWritePageView->NewPage(*iBlogs->GetCurrentBlog());
    SwitchView(EWritePageView);
}

void BlogView::ShowSinglePostView(WPPost post)
{
    qDebug("WP::BlogView::ShowSinglePostView()");
    
    iSinglePostView->ShowPost(*iBlogs->GetCurrentBlog(), post);
    SwitchView(ESinglePostView);
}

void BlogView::ShowEditPostView(WPPost post)
{
    qDebug("WP::BlogView::ShowEditPostView()");

    iAddPostView->EditPost(iBlogs->GetCurrentBlog(), post);
    SwitchView(EAddPostView);
}

void BlogView::ShowWriteNewPostView()
{
    iAddPostView->NewPost(iBlogs->GetCurrentBlog());
    SwitchView(EAddPostView);
}

void BlogView::ShowTabbedView()
{
    SwitchView(ETabbedView);
}

void BlogView::ShowBlogListView()
{
    // Update data so that latest activity is shown correctly
    iBlogView->UpdateData(*iBlogs);
    SwitchView(EBlogListView);
}

void BlogView::ShowNewBlogView()
{
    iSetupBlog->NewBlog();
    SwitchView(ESetupBlogView);
}

void BlogView::ShowEditBlogView(const QString& aBlogId)
{
    iSetupBlog->EditBlog(iBlogs->GetBlog(aBlogId));
    SwitchView(ESetupBlogView);
}

int BlogView::ShowApiKeyDialog() {
    return apiKeyDialog->exec();
}

void BlogView::ShowStatisticsView() {
    int code = QDialog::Accepted;
    if (!iNetworkEngine->HasApiKey()) {
        code = ShowApiKeyDialog();
    }

    if (code == QDialog::Accepted) {
        SwitchView(EStatisticsView);
    }
}

void BlogView::SetApiKey (const QString &apiKey)
{
    // show progress
    if (iNetworkEngine->SetApiKey(apiKey))
        // show progress for this
        iDialogController->StartTaskProgress();
}

void BlogView::SwitchView(const TMainView& aView)
{
    // Change the view
    //
    switch (aView)
    {
    case EInitialSetupBlogView:
    case EStatisticsView:
        iStackedWidget->setCurrentIndex(aView);
        //No action needed. All other "Views" uses Close() so this will be enabled by platform.
        break;
    case ECategoryView:
        ShowInLayout(iCategoryProxy);
        iStackedWidget->setCurrentIndex(ETabbedView);
        iTitleBar->SetText(iCategoryView->TitleText());
        iMenu->setButtonActions(iCategoryView->HappyAction(), iCategoryView->SadAction());
        iMenu->setMode(WPMenuGraphicsWidget::ESaveCancel);
        break;
    case EImageSelectionList:
        ShowInLayout(iImageSelectionProxy);
        iStackedWidget->setCurrentIndex(ETabbedView);
        iTitleBar->SetText(iImageSelectionList->TitleText());
        iMenu->setButtonActions(iImageSelectionList->HappyAction(), iImageSelectionList->SadAction());
        iMenu->setMode(WPMenuGraphicsWidget::ESaveCancel);
        break;
    case EBlogListView:
        ShowInLayout(iBlogListProxy);
        iStackedWidget->setCurrentIndex(ETabbedView);
        iTitleBar->SetText(iBlogView->TitleText());
        iMenu->setButtonActions(NULL, iBlogView->SadAction());
        iMenu->setMode(WPMenuGraphicsWidget::EBack);
        break;
    case ESinglePostView:
        ShowInLayout(iSinglePostProxy);
        iStackedWidget->setCurrentIndex(ETabbedView);
        iTitleBar->SetText(iSinglePostView->TitleText());
        iMenu->setButtonActions(iSinglePostView->HappyAction(), iSinglePostView->SadAction());
        iMenu->setMode(WPMenuGraphicsWidget::ESaveCancel);
        break;
    case ESetupBlogView:
        ShowInLayout(iSetupBlogProxy);
        iStackedWidget->setCurrentIndex(ETabbedView); // QGraphicsView must be foreground.
        iTitleBar->SetText(iSetupBlog->TitleText());
        iMenu->setButtonActions(iSetupBlog->HappyAction(), iSetupBlog->SadAction());
        iMenu->setMode(WPMenuGraphicsWidget::ESaveCancel);
        break;
    case EWritePageView:
        ShowInLayout(iTabView);
        iStackedWidget->setCurrentIndex(ETabbedView);
        iTitleBar->SetText(iWritePageView->TitleText());
        iMenu->setButtonActions(iWritePageView->HappyAction(), iWritePageView->SadAction());
        iMenu->setMode(WPMenuGraphicsWidget::ESaveCancel);
        iTabContentWidget->SwitchWidget(iWritePageProxy);
        //No action needed. All other "Views" uses Close() so this will be enabled by platform.
        break;
    case ETabbedView:
        ShowInLayout(iTabView);
        iStackedWidget->setCurrentIndex(ETabbedView);
        LaunchBlogView();
        break;
    case EAddPostView:
        ShowInLayout(iTabView);
        iStackedWidget->setCurrentIndex(ETabbedView);
        iTitleBar->SetText(iAddPostView->TitleText());
        iMenu->setButtonActions(iAddPostView->HappyAction(), iAddPostView->SadAction());
        iMenu->setMode(WPMenuGraphicsWidget::ESaveCancel);
        iTabContentWidget->SwitchWidget(iAddPostProxy);
        //No action needed. All other "Views" uses Close() so this will be enabled by platform.
        break;
    }

    setWindowTitle(iViewTitle.at(aView));
    //#else
    /*
    switch(aView)
    {
    case EInitialSetupBlogView:
        break;
    case ESetupBlogView:
        iSetupBlog->show();
        break;
    case EBlogListView:
        iBlogView->show();
        break;
    case EAddPostView:
        iAddPostView->show();
        break;
    case ESinglePostView:
        iSinglePostView->show();
        break;
    case EWritePageView:
        iWritePageView->show();
        break;
    case ECategoryView:
        iCategoryView->show();
        break;
    case EImageSelectionList:
        iImageSelectionList->show();
        break;
    case ETabbedView:
        //No action needed. All other "Views" uses Close() so this will be enabled by platform.
        break;
    case EStatisticsView:
        iStatisticsView->show();
    };

    // Check if we're swithing away from initial setup blog view. That view can't be
    // accessed again without restarting the application, so we'll delete it here.
    if (iCurrentView == EInitialSetupBlogView)
    {
        // QMainWindow has the ownership of the iInitialSetupBlog and
        // will take care of the deletion.
        setCentralWidget(iStackedWidget);
	}*/
    //#endif
    iCurrentView = aView;
    SetActiveView();
}

void BlogView::SaveBlogSetup(WPDataBlog *blog, const WPNetworkEndpoint &settings)
{
    // show progress for this
    iDialogController->StartTaskProgress();

    if (blog)
        iBlogs->EditBlog(blog, settings);
    else
        iBlogs->NewBlog(settings);
}

void BlogView::DeletePage(TextListItem* aItem)
{
    WPDataBlog &blog = *iBlogs->GetCurrentBlog();
    WPPage page = blog.iPages.GetByLocalId(aItem->ItemId());

    if (page)
        page->Delete();
}

void BlogView::DeletePost(WPPost post)
{
    qDebug("WP::BlogView::DeletePost()");

    post->Delete();
}

void BlogView::OpenBlog(const QString& aBlogId)
{
    // show progress for this
    iDialogController->StartTaskProgress();

    iBlogs->OpenBlog(iBlogs->GetBlog(aBlogId));

    ShowTabbedView();
}

bool BlogView::DeleteBlog(const QString& aBlogId)
{
    return iBlogs->DeleteBlog(iBlogs->GetBlog(aBlogId));
}

void BlogView::ShowCategoryView(const QStringList* aCategories)
{
    iCategoryView->SelectCategories(aCategories);
    SwitchView(ECategoryView);
}

void BlogView::SelectCategories(const QStringList* aCategories)
{
    iAddPostView->SelectCategories(aCategories);
    SwitchView(EAddPostView);
}

void BlogView::ShowImageList(const QStringList& aInitialSelections, bool aPostView)
{
    iAddPhotosToPosts = aPostView;
//#ifndef Q_WS_MAEMO_5
/*    if (iImageSelectionProxy->widget())
    {
        delete iImageSelectionList;
        iImageSelectionList = NULL;
    }
  */  
//#endif
/*#ifdef Q_WS_MAEMO_5
    if (aPostView)
    {
        iImageSelectionList = new ImageSelectionList(aInitialSelections, iAddPostView);
    }
    else
    {
        iImageSelectionList = new ImageSelectionList(aInitialSelections, iWritePageView);
    }
#else*/
    ImageSelectionList* tmp = iImageSelectionList;
    iImageSelectionList = new ImageSelectionList(aInitialSelections, this);
    iImageSelectionProxy->setWidget(iImageSelectionList);
    delete tmp;
    //iStackedWidget->insertWidget(EImageSelectionList, iImageSelectionList);
//#endif

    connect(iImageSelectionList, SIGNAL(ImagesSelected(QStringList)), this, SLOT(ImagesSelected(QStringList))); // TODO change to function call, no point to use connect in this close classes
    connect(
            iImageSelectionList, SIGNAL(TaskStarted(TWPNetworkEngineCommand)), 
            iDialogController, SLOT(TaskStarted(TWPNetworkEngineCommand)));

    connect(
            iImageSelectionList, SIGNAL(TaskFinished(TWPNetworkEngineCommand)), 
            iDialogController, SLOT(TaskFinished(TWPNetworkEngineCommand))
    );

    // Call this only after the connections so that we can show a progess dialog
    iImageSelectionList->SearchImages();
    SwitchView(EImageSelectionList);
}

void BlogView::ImagesSelected(QStringList aFilenames)
{
    if (iAddPhotosToPosts)
    {
        iAddPostView->ImagesAdded(aFilenames);
        SwitchView(EAddPostView);
    }
    else
    {
        iWritePageView->ImagesAdded(aFilenames);
        SwitchView(EWritePageView);
    }
}

void BlogView::AddNewComment(WPPost aPost, const QString& aBody)
{
    aPost->NewComment(aBody);

    SwitchView(ESinglePostView);
}

void BlogView::ReplyToComment(WPComment aComment)
{
    WPDataBlog *blog = iBlogs->GetCurrentBlog();

    WPPost post = blog->iPosts.GetByItemId(aComment->iPostId);

    if (post) {
        iSinglePostView->ShowPost(*blog, post);
        SwitchView(ESinglePostView);
    }
}

void BlogView::Exit()
{
    QCoreApplication::exit(0);
}

void BlogView::About()
{
    // TODO add an icon
    iDialogController->ShowInfoNote("About", "Nokia WordPress\nversion "); //+ QString(WP_VERSION));
}

void BlogView::Refresh()
{
    // show progress dialog
    iDialogController->StartTaskProgress();
    iBlogs->RefreshCurrentBlog();
}

/**
 * Sets the active view after ProgressDialog closes. 
 */
void BlogView::SetActiveView(BlogView::TMainView aRefocusableView)
{
  //#ifndef Q_WS_MAEMO_5
    // Update focus if
    //   a) there is an active view AND
    //   b) either the call is general, or a specific view wants its focus
    //      to be updated in case it is the current view (separate views do
    //      not know if they are active at a given time or not).
    if  (iStackedWidget->currentWidget() && 
         (aRefocusableView == BlogView::ENoView || aRefocusableView == iCurrentView))
    {
        if (iCurrentView == EStatisticsView)
        {
            showMaximized();
        }
        else
        {
            showFullScreen();
        }
        // TODO non-virtual setFocus calls base class function if called from
        // QWidget* -- create some Focusable interface to fix this? For now,
        // call separately for each view that has special focus function.
        switch (iCurrentView)
        {
            case ETabbedView:
                if (iTabContentWidget->CurrentWidget() == iPannableCommentsWidget)
                {
                    iCommentsView->setFocus();
                } 
                else if (iTabContentWidget->CurrentWidget() == iPannablePostsWidget)
                {
                    iPostsView->setFocus();
                }
                else if (iTabContentWidget->CurrentWidget() == iPannablePagesWidget)
                {
                    iPagesView->setFocus();
                }
                break;
            case ECategoryView:
                iCategoryView->setFocus();
                break;
            case ESetupBlogView:
                iSetupBlog->setFocus();
                break;
            case EBlogListView:
                iBlogView->setFocus();
                break;
            case EAddPostView:
                iAddPostView->setFocus();
                break;
            case EWritePageView:
                iWritePageView->setFocus();
                break;
            case EImageSelectionList:
                if (iImageSelectionProxy->widget())
                    iImageSelectionProxy->widget()->setFocus();
                break;
            case EStatisticsView:
                iStatisticsView->setFocus();
                break;
            default:
                iStackedWidget->currentWidget()->setFocus();
                break;
        }

        // After calling setFocus, the options menu receives an extra "Actions" item.
        // It's not BlogView, it's not currentView. We'll get around it for now
        // by disabling context menu in all widgets.
        QList<QWidget*> list = QApplication::allWidgets();
        const int count = list.count();
        for (int i = 0; i < count; i++)
        {
            list[i]->setContextMenuPolicy(Qt::NoContextMenu);
        }
    }
    //#endif
}

void BlogView::ActivateBusyIndicator() 
{ 
    iBusyIndicator->Activate();
}
void BlogView::DeactivateBusyIndicator() 
{ 
    iBusyIndicator->Deactivate(); 
}

BusyIndicator& BlogView::GetBusyIndicator()
{
    return *iBusyIndicator;
}

void BlogView::resizeEvent(QResizeEvent *e)
{
    qDebug() << "WP::BlogView::resizeEvent " << e->size().width() << " x " << e->size().height() << endl;
    QSize window = QApplication::desktop()->screenGeometry().size();

    // In portrait Symbian phones, let's use smaller status pane than normal
    // to get more pixels for the content
    if (!iPortrait && window.width() < window.height()) // Portrait mode
    {
        iHorizontalLayout->removeItem(iMenu);
        iMenu->setMaximumHeight(MenuHeight);
        iLayout->addItem(iMenu);
        iPortrait = true;
//        iHorizontalLayout->addItem(iLayout);
//        iView->setScene(iScene);
    }
    else if (iPortrait && window.width() > window.height())
    {
        iLayout->removeItem(iMenu);
        iMenu->setMaximumHeight(window.height());
        iHorizontalLayout->addItem(iMenu);
        iPortrait = false;
    }

    // Workaround for landscape mode height exceeding the available height.
    // The pixel amount is hardcoded, found out manually.
    // TODO If a Qt fix becomes available, disable this.
    if (!iLandscapeSizeWorkaround)
    {
        // The maximum height is only set in the first resize event in landscape mode.
        // This seems to be the only situation when the application gets the wrong height.
        if (window.width() == 640 && window.height() == 360)
            iStackedWidget->setMaximumHeight(geometry().size().height()); // 29 -- 85
        else if (window.width() == 320 && window.height() == 240)
            iStackedWidget->setMaximumHeight(geometry().size().height()); // 57 -- 107
    }
    else if (iStackedWidget->maximumHeight() < QWIDGETSIZE_MAX)
    {
        // In further resize events, clear all restrictions
        iStackedWidget->setMaximumHeight(QWIDGETSIZE_MAX);
    }

    iLandscapeSizeWorkaround = true;
    e->ignore();
    if (iBusyIndicator)
        iBusyIndicator->resize(e->size());
    if (iScene && iLayoutWidget)
    {
        iLayoutWidget->resize(window.width(),window.height());
        iSetupBlogProxy->setMaximumWidth(window.width());
    }
}

bool BlogView::keyClickedEvent(QKeyEvent* event)
{
    keyReleaseEvent(event);
    if (event->isAccepted())
        return true;
    else
        return false;
}
