/*
 * BaseView.h
 *
 *  Created on: 5.11.2009
 *      Author: thyv
 */

#ifndef BLOGVIEW_H_
#define BLOGVIEW_H_

#include <QMainWindow>
#include <QGraphicsView>
#include <QDebug>
#include "WPData.h"
#include "busyindicator.h"
#include "themedatastorage.h"
#include "wpkeydispatcher.h"

class QGraphicsScene;
class QGraphicsView;
class QGraphicsLinearLayout;
class QGraphicsWidget;

class QListWidgetItem;
class QStackedWidget;
class QTabWidget;
class QMenuBar;
class QAction;
class QFileSystemModel;

class WritePageView;
class AddPostView;
class SinglePostView;
class CategoryView;
class StatisticsView;

class PostsView;
class PagesView;
class CommentsView;
class CommentsGraphicsView;
class SetupBlog;
class BlogSetupView;
class WPNetworkEngine;
class TextListItem;
class ImageSelectionList;
class DialogController;
class ApiKeyDialog;
class WPTabbarGraphicsView;
class PannableView;
class WPMenuGraphicsWidget;
class WPStackedGraphicsWidget;
class WPTitleBarGraphicsWidget;

class BlogView : public QMainWindow, public WPKeyDispatcher
{

    Q_OBJECT

public:
    BlogView();
    ~BlogView();

    // The order is significant
    enum TMainView
    {
        EInitialSetupBlogView = -2, // Maemo specific, so the index doesn't have to be positive
        ENoView               = -1, // Not used anymore
        EStatisticsView       =  0,
        ETabbedView           =  1, // At the ends this is the only widget and stacked widget can be thrown away.
        EImageSelectionList   =  2, // This should be the last one as it is created dynamically
        ECategoryView         =  3, // this is here just because for references, to be deleted.
        EBlogListView         =  4, // this is here just because for references, to be deleted.
        EAddPostView          =  5, // this is here just because for references, to be deleted.
        EWritePageView        =  6, // this is here just because for references, to be deleted.
        ESetupBlogView        =  7, // this is here just because for references, to be deleted.
        ESinglePostView       =  8  // this is here just because for references, to be deleted.
//        ECategoryView         =  0,
//       EBlogListView         =  0,
//        ESetupBlogView        =  0,
//        ESinglePostView       =  1,
//        EWritePageView        =  2,
//        EAddPostView          =  3,
//        ESinglePostView       =  4,
//        ECategoryView         =  5,
//        EStatisticsView       =  6,
//        ETabbedView           =  7,
//        EImageSelectionList   =  8 // This should be the last one as it is created dynamically
                             };

    QStringList iViewTitle;

    void SwitchView(const TMainView& aView);
    void SaveBlogSetup(WPDataBlog *blog, const WPNetworkEndpoint &settings);

    void AddNewComment(WPPost aPost, const QString& aBody);
    void ShowCategoryView(const QStringList* aCategories);
    void SelectCategories(const QStringList* aCategories);
    void AddNewCategory(const QString& aCategoryName);
    void ShowImageList(const QStringList& aInitialSelections, bool aPostView); // Pass true if post view, false if pages view
    void SetActiveView(BlogView::TMainView aRefocusableView = BlogView::ENoView);

    void ActivateBusyIndicator();
    void DeactivateBusyIndicator();
    BusyIndicator& GetBusyIndicator();
signals:
    void SignalBlogListView();

public slots:
    void ShowTabbedView();
    void Exit();
    void Refresh();

    // Pages
    void ShowEditPageView(WPPage aPage);
    void ShowWriteNewPageView();
    void DeletePage(TextListItem* aItem);
    
    // Posts
    void ShowEditPostView(WPPost post);
    void ShowSinglePostView(WPPost post);
    void ShowWriteNewPostView();
    void DeletePost(WPPost post);
    void ReplyToComment(WPComment aComment);
    void ImagesSelected(QStringList aFilenames);

    // Blogs
    void ShowBlogListView();
    void ShowNewBlogView();
    void ShowEditBlogView(const QString& aBlogId);
    void OpenBlog(const QString& aBlogId);
    bool DeleteBlog(const QString& aBlogId);

    // Statistics
    void ShowStatisticsView();
    int ShowApiKeyDialog();
    void SetApiKey (const QString &apiKey);
    
    inline ThemeData& GetThemeData() { return *iThemeData; }

public: // New functions
    WPNetworkEngine* GetNetworkEngine() const
    {
        return iNetworkEngine;
    }

    WPDataGravatars* GetGravatars() const
    {
        return iGravatars;
    }

    void keyReleaseEvent( QKeyEvent * event  );
    
private:
    void LaunchBlogView();
    void SetSoftKeys();
    void ShowInLayout(QGraphicsWidget* aItem);
    bool TabsActive();

private slots:
    void About();
    void resizeEvent(QResizeEvent *e);

private: // WPKeyDispatcher
    bool keyClickedEvent(QKeyEvent* event);
    
private:
    // Views
    QStackedWidget* iStackedWidget;
    WPTitleBarGraphicsWidget* iTitleBar;
    WPTabbarGraphicsView* iTabView;
    WPMenuGraphicsWidget* iMenu;
    PostsView* iPostsView;
    PannableView* iPannablePostsWidget;
    PagesView* iPagesView;
    PannableView* iPannablePagesWidget;

    CommentsView* iCommentsView;
    PannableView* iPannableCommentsWidget;
    WPStackedGraphicsWidget* iTabContentWidget;
    
    SetupBlog* iSetupBlog;
    QGraphicsProxyWidget* iSetupBlogProxy;
    
    QGraphicsProxyWidget* iBlogListProxy;
    BlogSetupView* iBlogView;
#ifdef Q_WS_MAEMO_5
    SetupBlog* iInitialSetupBlog;
#endif
    QGraphicsProxyWidget* iWritePageProxy;
    WritePageView* iWritePageView;

    QGraphicsProxyWidget* iAddPostProxy;
    AddPostView* iAddPostView;

    QGraphicsProxyWidget* iSinglePostProxy;
    SinglePostView* iSinglePostView;
    
    QGraphicsProxyWidget* iImageSelectionProxy;
    ImageSelectionList* iImageSelectionList;
    
    QGraphicsProxyWidget* iCategoryProxy;
    CategoryView* iCategoryView;
    
    StatisticsView* iStatisticsView;
    DialogController* iDialogController;

    ApiKeyDialog* apiKeyDialog;

    QGraphicsScene*         iScene;
    QGraphicsView*          iView;
    QGraphicsLinearLayout*  iLayout;
    QGraphicsLinearLayout*  iHorizontalLayout;
    QGraphicsWidget*        iLayoutWidget;
    bool                    iPortrait;

    // Model/Network
    WPNetworkEngine* iNetworkEngine;
    WPDataUsersBlogs* iBlogs;
    WPDataGravatars* iGravatars;
    
    int iCurrentView;
    bool iLandscapeSizeWorkaround;
    bool iAddPhotosToPosts; // True if post view, false if pages view
    
    BusyIndicator* iBusyIndicator;
    
    QAction* iManageBlogs;
    QAction* iActionWritePage; 
    QAction* iActionWritePost;
    QAction* iActionShowStatistics; 
    QAction* iActionRefresh;
    QAction* iAbout;
    QAction* iExit;
    ThemeData* iThemeData;
};
#endif //BLOGVIEW_H_
