
#include <QSvgWidget>
#include <QPixmap>

#include "commentsview.h"
#include "WPHomeScreenInfo.h"
#include "WPData.h"

#ifdef Q_OS_SYMBIAN
static const int KItemDefaultHeight = 123;
static const int KLogoDefaultSize = 75;
#else
static const int KItemDefaultHeight = 170;
static const int KLogoDefaultSize = 115;
#endif

WPHomeScreenInfo::WPHomeScreenInfo(CommentsView& aParent) : QFrame(NULL), iParent(aParent)
{
    ui.setupUi(this);

    QSvgWidget *svg = new QSvgWidget(":/images/wp-logo.svg", this);
    svg->setFixedSize(KLogoDefaultSize, KLogoDefaultSize);
    ui.logoLayout->addWidget(svg);
    setFixedHeight(KItemDefaultHeight);
}

WPHomeScreenInfo::~WPHomeScreenInfo()
{
}

void WPHomeScreenInfo::UpdateData(const WPDataBlog *aData)
{
    WPDataComment comment;
    const int comments = aData->iComments.size();
    int unapproved = 0;
    foreach (comment, aData->iComments)
    {
        if (comment.iStatus == QLatin1String("hold"))
            unapproved++;
    }

    QString statsText;
    statsText.reserve(128);
    statsText += tr("%1 unapproved comments").arg(unapproved) + '\n';
    statsText += tr("%1 comments in %2 posts").arg(comments).arg(aData->iPosts.size()) + '\n';
    if (!aData->iApiKey.isEmpty() && aData->iStats.iLastUpdated == QDate::currentDate()) {
        statsText += QString::number(aData->iStats.iTotalVisits);
        statsText += tr(" visits in last 365 days");
    }

    ui.blogName->setText(aData->iBlogName);
    ui.statsLabel->setText(statsText);
}

void WPHomeScreenInfo::resizeEvent(QResizeEvent* aEvent)
{
    // Set max size for the clipping label -- otherwise it will start maximized and decrease in small steps
    // WP logo = 75
    // Padding on both sides = 4
    const int modifier = 75 + 4;
    const int width = iParent.VisibleSize().width() - modifier;
    ui.blogName->setMaximumWidth(width);
    QFrame::resizeEvent(aEvent);
}
