/*
 * statisticsview.cpp
 *
 *  Created on: 15.3.2010
 *      Author: aahv
 */

#include "statisticsview.h"
#include "ui_statisticsview.h"

#include <QList>
#include <QPair>
#include <QDate>
#include <QMap>
#include <QPushButton>

StatisticsView::StatisticsView(BlogView *parent) :
        QFrame(parent),
        ui(new Ui::StatisticsView), iParent(parent)
{
#ifdef Q_WS_MAEMO_5
    setWindowFlags(Qt::Window);
    setAttribute(Qt::WA_Maemo5StackedWindow);
#endif
    ui->setupUi(this);

    ui->referrers->setListView(Referrers);
    ui->searches->setListView(Searches);

    errorArea = new WPScrollArea(this);
    errorLabel = new QLabel(this);
    errorLabel->setAlignment(Qt::AlignTop);
    errorLabel->setWordWrap(true);
    errorArea->addWidget(errorLabel);
    QPushButton* enterButton = new QPushButton(tr("Re-enter API key"), this);
    errorArea->addWidget(enterButton);
    ui->verticalLayout->addWidget(errorArea);
    errorArea->setVisible(false);
    connect(enterButton, SIGNAL(clicked()), iParent, SLOT(ShowApiKeyDialog()));

#ifdef Q_OS_SYMBIAN
    ui->tabWidget->setTabPosition(QTabWidget::North);
    ui->tabWidget->setTabText(0, tr("Visits"));
    ui->tabWidget->setTabText(1, tr("Referrers"));
    ui->tabWidget->setTabText(2, tr("Searches"));
#else
    ui->tabWidget->setTabPosition(QTabWidget::West);
    ui->tabWidget->setTabText(0, tr("  Visits  "));
    ui->tabWidget->setTabText(1, tr("  Referrers  "));
    ui->tabWidget->setTabText(2, tr("  Searches  "));
#endif

#ifdef Q_OS_SYMBIAN
    QAction* closeAction = new QAction(tr("Close"), this);
    connect(closeAction, SIGNAL(triggered()), iParent, SLOT(ShowTabbedView()));
    closeAction->setSoftKeyRole(QAction::NegativeSoftKey);
    addAction(closeAction);
#else
#ifndef Q_WS_MAEMO_5
    QPushButton* closeButton = new QPushButton(tr("Close"));
    closeButton->setObjectName(QString::fromLatin1("Maemo5NavigationButton"));
    connect(closeButton, SIGNAL(clicked()), iParent, SLOT(ShowTabbedView()));

    QHBoxLayout* buttonLayout = new QHBoxLayout();
    buttonLayout->addWidget(closeButton);

    ui->verticalLayout->addLayout(buttonLayout);
#endif
#endif
}

StatisticsView::~StatisticsView()
{
    delete ui;
}

void StatisticsView::focusInEvent(QFocusEvent* event) {
    QFrame::focusInEvent(event);
#ifdef Q_OS_SYMBIAN
    if (ui->tabWidget->isVisible()) {
	switch(ui->tabWidget->currentIndex()) {
        case 0:
            ui->visits->setFocus();
            break;
        case 1:
            ui->referrers->setFocus();
            break;
        case 2:
            ui->searches->setFocus();
            break;
	}
    } else {
        errorArea->setFocus();
    }
#endif
}

void StatisticsView::changeEvent(QEvent *e)
{
    QWidget::changeEvent(e);
    switch (e->type()) {
    case QEvent::LanguageChange:
        ui->retranslateUi(this);
        break;
    default:
        break;
    }
}

void StatisticsView::updateStats(const WPDataStats &stats) {
    errorArea->setVisible(false);
    if (stats.iLastUpdated == QDate::currentDate()) {
        ui->visits->updateStats(stats);
        ui->referrers->updateStats(stats);
        ui->searches->updateStats(stats);
    }
    ui->tabWidget->setVisible(true);
}

void StatisticsView::updateStatsFailed(const QString &error) {
    ui->tabWidget->setVisible(false);
    errorLabel->setText(error);
    errorArea->setVisible(true);
}
