#ifndef WPCOMMENTGRAPHICSLIST_H
#define WPCOMMENTGRAPHICSLIST_H

#include <QGraphicsWidget>
#include <QGraphicsView>
#include <QList>

#include "busyindicator.h"
#include "themedatastorage.h"
#include "WPData.h"

class QLabel;
class QVBoxLayout;
class QPropertyAnimation;

class PopupMenu;
class CommentsGraphicsView;
class WPCommentGraphicsListItem;
class WPNetworkEngine;
class QGraphicsProxyWidget;
class QGraphicsLinearLayout;
class PannableView;
class BlogView;
class WPKeyDispatcher;

class WPCommentGraphicsList : public QGraphicsWidget
{
    Q_OBJECT

public:
    WPCommentGraphicsList(QListIterator<WPComment> aData, BlogView* aBlogView, const QGraphicsView& aGraphicsView,
        PannableView* aPannableView, bool aShowFullComment = false);
    ~WPCommentGraphicsList();

public:
    const QFont& titleFont() const { return iTitleFont; };
    const QFont& contentFont() const { return iContentFont; };
    const QFont& normalFont() const { return iNormalFont; };

signals:
    void ReplyToComment(WPComment aComment);

public slots:
    void GravatarUpdated (const QString &email, const QPixmap &pixmap);
    void ShowContextMenu(WPCommentGraphicsListItem* aItem, QPoint aPos);
    void ContextMenuHidden();
    
    void CommentChanged(WPComment aComment);
    void CommentRemoved(WPComment aComment);
    void CommentAdded(WPComment aComment);

    void setFocus();

protected:
    void resizeEvent(QGraphicsSceneResizeEvent* event);
    QPoint getMenuLocation(QPoint itemPoint);
    void keyPressEvent( QKeyEvent * event  );
    void keyReleaseEvent( QKeyEvent * event );
    
private slots:
    void MenuHiding();
    void Approve();
    void Unapprove();
    void View();
    void Reply();
    void Delete();
    void Spam();

private:
    void AddComment(WPComment aComment, bool aShowFullComment);
    void RemoveComment (WPComment aComment);
    
    void SetLabelText();
    
    void DecrementUnapproved();
    void DecrementApproved();
    
    void IncrementUnapproved();
    void IncrementApproved();

private: // Data
    WPDataGravatars*        iGravatars;
    const QGraphicsView&    iGraphicsView;
    WPKeyDispatcher*        iKeyDispatcher;

    QList<WPCommentGraphicsListItem*> iItems;
    WPCommentGraphicsListItem* iCurrentItem;
    QGraphicsLinearLayout* iLayout;
    
    QLabel* iUnapprovedLabel;
    QGraphicsProxyWidget* iUnapprovedProxy;
    QLabel* iApprovedLabel;
    QGraphicsProxyWidget* iApprovedProxy;
    int iUnapproved;
    int iApproved;

    QAction* iActionApprove;
    QAction* iActionUnapprove;
    QAction* iActionView;
    QAction* iActionHide;
    QAction* iActionReply;
    QAction* iActionDelete;
    QAction* iActionSpam;

    PopupMenu* iPopupMenu;
    QPropertyAnimation* iAnimation;

    QGraphicsProxyWidget* iMenuProxy;
    
    BusyIndicator& iBusyIndicator;
    QFont iTitleFont;
    QFont iContentFont;
    QFont iNormalFont;
    bool  iMenuHiding;
    ThemeData& iThemeData;
    PannableView* iPannableView;
};

#endif // WPCOMMENTGRAPHICSLIST_H
