#ifndef WPCOMMENTGRAPHICSLISTITEM_H
#define WPCOMMENTGRAPHICSLISTITEM_H

#include <QKeyEvent>
#include <QGraphicsWidget>

#include "WPData.h"
#include "themedatastorage.h"

class QPixmap;
class WPCommentGraphicsList;
class BusyIndicator;

class WPCommentGraphicsListItem : public QGraphicsWidget
{
    Q_OBJECT

public:
    WPCommentGraphicsListItem(const WPCommentGraphicsList& aList, WPComment aComment, 
        bool aShowFullComment, BusyIndicator& aBusyIndicator, ThemeData& aThemeData);
    ~WPCommentGraphicsListItem();
    
public: // New functions
    void SetAvatarImage(const QPixmap &aPixmap);
    WPComment GetComment() const;
    QPointF TopRight() const;
    void ChangeViewMode();
    bool ShowFullComment() const { return iShowFullComment; };
    void Update (WPComment aComment);
    
    QRectF boundingRect() const;
    void paint(QPainter* painter, const QStyleOptionGraphicsItem* option, QWidget* widget = 0);
    bool IsBusy();
    void MarkBogusFocus(bool aActive);
    bool BogusFocus();

public slots:
    void resizeEvent(QGraphicsSceneResizeEvent *e);

signals:
    void CommentItemSelected(WPCommentGraphicsListItem* aItem, QPoint aPos);

private:
    void mousePressEvent(QGraphicsSceneMouseEvent *e);
    void mouseReleaseEvent(QGraphicsSceneMouseEvent *e);
    void keyPressEvent(QKeyEvent *e);
    void keyReleaseEvent(QKeyEvent *e);
    
    void calculateElidedTexts();
    void timerEvent(QTimerEvent*);
    void SetViewMode(bool aShowFullComment);
private: // Data
    const WPCommentGraphicsList& iList;
    WPComment iComment;
    bool iShowFullComment;
    bool iKeyPressReceived;
    qreal iWidth;
    qreal iHeight;
    int iBusyX;
    int iBusyY;
    int iPicWidth;
    int iPicHeight;
    QRectF iBoundingRect;
    int iCol1ContentXOffset;
    int iCol2ContentXOffset;
    int iTimerId;
    BusyIndicator& iBusyIndicator;
    QPixmap iPixmap;
    QString iDate;
    QString iTitle;
    QString iCommentText;
    bool iHasMenuFocus;
    ThemeData& iThemeData;
    QRect iTitleSize;
    QRect iCommentSize;
    
    QPixmap* iCommentItemPixmap;
};

#endif // WPCOMMENTGRAPHICSLISTITEM_H
