#ifndef WPPageGraphicsListItem_H
#define WPPageGraphicsListItem_H

#include <QKeyEvent>
#include <QGraphicsWidget>

#include "WPData.h"
#include "wppagegraphicslist.h"

class QPixmap;
class BusyIndicator;

class WPPageGraphicsListItem : public QGraphicsWidget
{
    Q_OBJECT

public:
    WPPageGraphicsListItem(const WPPageGraphicsList& aList, WPPage aPage, BusyIndicator& aBusyIndicator, ThemeData& aThemeData);
    ~WPPageGraphicsListItem();
    
public: // New functions
    QPointF TopRight() const;
    void ChangeViewMode();
    void Update(const WPDataComment& aComment);
    
    QRectF boundingRect() const;
    void paint(QPainter* painter, const QStyleOptionGraphicsItem* option, QWidget* widget = 0);
    bool IsBusy();
    void MarkBogusFocus(bool aActive);
    bool BogusFocus();

public:
    static const int ItemMinimumHeight = 145;

public slots:
    void resizeEvent(QGraphicsSceneResizeEvent *e);
    WPPage GetPage();
    
signals:
    void PageItemSelected(WPPageGraphicsListItem* aItem, QPoint aPos);

private:
    void mousePressEvent(QGraphicsSceneMouseEvent *e);
    void mouseReleaseEvent(QGraphicsSceneMouseEvent *e);
    void keyPressEvent(QKeyEvent *e);
    void keyReleaseEvent(QKeyEvent *e);
    
    void calculateElidedTexts();
    void timerEvent(QTimerEvent*);

private: // Data
    const WPPageGraphicsList& iList;
    WPPage iPage;
    bool iKeyPressReceived;
    qreal iWidth;
    int iBusyX;
    int iBusyY;
    int iPicWidth;
    QRectF iBoundingRect;
    int iCol1ContentXOffset;
    int iTimerId;
    BusyIndicator& iBusyIndicator;
    QPixmap iPixmap;
    QString iDate;
    QString iTitle;
    bool iHasMenuFocus;
    ThemeData& iThemeData;
};

#endif // WPPageGraphicsListItem_H
