/*
 * wppostgraphicslist.h
 *
 *  Created on: 19.5.2010
 *      Author: jlep
 */

#ifndef WPPOSTGRAPHICSLIST_H_
#define WPPOSTGRAPHICSLIST_H_

#include <QGraphicsWidget>
#include <QGraphicsView>
#include <QList>

#include "WPData.h"
#include "busyindicator.h"
#include "themedatastorage.h"

class QLabel;
class QPushButton;
class QPropertyAnimation;
class WPPostGraphicsListItem;

class PopupMenu;
class PostGraphicsView;
class WPPostGraphicsListItem;
class QGraphicsProxyWidget;
class QGraphicsLinearLayout;
class PannableView;
class BlogView;
class WPKeyDispatcher;

class WPPostGraphicsList  : public QGraphicsWidget
{
    Q_OBJECT

public:
    enum TMenuAction
    {
    	EOpen,
    	EEdit,
    	EDelete
    };

    WPPostGraphicsList (QListIterator<WPPost> aPosts, const QGraphicsView& aGraphicsView, 
        BlogView* aBlogView, PannableView* aPannableView);
    ~WPPostGraphicsList();

public:
    const QFont& titleFont() const { return iTitleFont; };
    const QFont& authorFont() const { return iAuthorFont; };
    const QFont& infoFont() const { return iInfoFont; };
    void setFocus();

public slots:
    void PostChanged(WPPost aPost);
    void PostRemoved(WPPost aPost);
    void PostAdded(WPPost aPost);
    
signals:
    void OpenClicked(WPPost aPost);
    void EditClicked(WPPost aPost);
    void DeleteClicked(WPPost aPost);
    
private:
    void AddPost(WPPost aPost);
    void RemovePost (WPPost aPost);

    void SetLabelText();
    
    void DecrementDrafts();
    void DecrementPosts();
    
    void IncrementDrafts();
    void IncrementPosts();

    void resizeEvent(QGraphicsSceneResizeEvent *e);

private slots:
    void Open();
    void Edit();
    void Delete();

public slots:
    void MenuHiding();
    void ShowContextMenu(WPPostGraphicsListItem* aItem, QPoint aPos);
    void ContextMenuHidden();

protected:
   QPoint getMenuLocation(QPoint itemPoint);
   void keyPressEvent( QKeyEvent * event  );
   void keyReleaseEvent( QKeyEvent * event );

   
private: // Data
    QString iTitle;
    WPNetworkEngine& iNetworkEngine;
    const QGraphicsView& iGraphicsView;
    WPKeyDispatcher*        iKeyDispatcher;

    QList<WPPostGraphicsListItem*> iItems;
    WPPostGraphicsListItem* iCurrentItem;
    QGraphicsLinearLayout* iLayout;
    
    QLabel* iDraftsLabel;
    QGraphicsProxyWidget* iDraftsProxy;
    QLabel* iPostsLabel;
    QGraphicsProxyWidget* iPostsProxy;
    int iDrafts;
    int iPosts;

    
    QAction* iActionOpen;
    QAction* iActionEdit;
    QAction* iActionDelete;
    int iStretchCount;
    PopupMenu* iPopupMenu;
    QPropertyAnimation* iAnimation;
    
    QPushButton* iNewPostButton;
    QGraphicsProxyWidget* iNewPostProxy;
    
    QGraphicsProxyWidget* iMenuProxy;
    
    BusyIndicator& iBusyIndicator;
    QFont iTitleFont;
    QFont iAuthorFont;
    QFont iInfoFont;
    bool  iMenuHiding;
    ThemeData& iThemeData;
    PannableView* iPannableView;
};

#endif /* WPPOSTGRAPHICSLIST_H_ */
