#ifndef WPPostGraphicsListItem_H
#define WPPostGraphicsListItem_H

#include <QKeyEvent>
#include <QGraphicsWidget>

#include "WPData.h"
#include "wppostgraphicslist.h"

class QPixmap;
class BusyIndicator;

class WPPostGraphicsListItem : public QGraphicsWidget
{
    Q_OBJECT

public:
    WPPostGraphicsListItem(const WPPostGraphicsList& aList, WPPost aPost, BusyIndicator& aBusyIndicator, ThemeData& aThemeData);
    ~WPPostGraphicsListItem();
    
public: // New functions
    QPointF TopRight() const;
    void ChangeViewMode();
    
    QRectF boundingRect() const;
    void paint(QPainter* painter, const QStyleOptionGraphicsItem* option, QWidget* widget = 0);
    bool IsBusy();
    void MarkBogusFocus(bool aActive);
    bool BogusFocus();

public:
    static const int ItemMinimumHeight = 145;

public slots:
    void resizeEvent(QGraphicsSceneResizeEvent *e);
    WPPost GetPost() const;
    
signals:
    void PostItemSelected(WPPostGraphicsListItem* aItem, QPoint aPos);

private:
    void mousePressEvent(QGraphicsSceneMouseEvent *e);
    void mouseReleaseEvent(QGraphicsSceneMouseEvent *e);
    void keyPressEvent(QKeyEvent *e);
    void keyReleaseEvent(QKeyEvent *e);
    
    void calculateElidedTexts();
    void timerEvent(QTimerEvent*);
private: // Data
    const WPPostGraphicsList& iList;
    WPPost iPost;
    bool iKeyPressReceived;
    qreal iWidth;
    int iBusyX;
    int iBusyY;
    int iPicWidth;
    QRectF iBoundingRect;
    int iCol1ContentXOffset;
    int iTimerId;
    BusyIndicator& iBusyIndicator;
    QPixmap iPixmap;
    QString iDate;
    QString iTitle;
    bool iHasMenuFocus;
    ThemeData& iThemeData;
};

#endif // WPPostGraphicsListItem_H
